//
//  DTDateParsing.m
//  Pantomime
//
//  Created by Annard Brouwer on 07/10/2008.
//  Copyright 2008 DEVONtechnologies, LLC. All rights reserved.
//

#import <Ruby/ruby.h>

#import "DTDateParsing.h"

static DTDateParsing *_defaultDateParser;

static VALUE wrap_conversion(VALUE args);
static VALUE protected_conversion(VALUE conversionObject, const char *dateStr);

@implementation DTDateParsing

+ (id)defaultDateParser
{
    if (nil == _defaultDateParser)
    {
        NSString *rbResourcePath;
        
        rbResourcePath = [[NSBundle bundleForClass: self] pathForResource: @"DTDateParsingHelper" ofType: @"rb"];
        NSAssert(nil != rbResourcePath, @"Can't load Ruby script from bundle!");
        ruby_init();
        ruby_init_loadpath();
        ruby_script("embedded");
        rb_require([rbResourcePath fileSystemRepresentation]);
        _defaultDateParser = [[self alloc] init];
    }
    return _defaultDateParser;
}

+ (void)clearDefaultParser
{
    [_defaultDateParser release];
    _defaultDateParser = nil;
    ruby_finalize();
}

- (id)init
{
    [super init];
    dateHelperInstance = rb_class_new_instance(0, 0, rb_const_get(rb_cObject, rb_intern("DTDateParsingHelper")));    
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (NSDate *)parseDateFromString:(NSString *)aDateStr
{
    VALUE result;
    NSString *newDateStr;
    
    result = protected_conversion(dateHelperInstance, [aDateStr UTF8String]);
    if (NIL_P(result))
        return nil;
    
    newDateStr = [NSString stringWithCString: StringValuePtr(result)
                                    encoding: NSUTF8StringEncoding];
    return [NSDate dateWithString: newDateStr];
}

@end

static VALUE wrap_conversion(VALUE args)
{
    VALUE *values = (VALUE *)args;
    VALUE obj = values[0];
    VALUE param = values[1];

    return rb_funcall(obj, rb_intern("convertToInternationalDateString"), 1, param);
}

static VALUE protected_conversion(VALUE conversionObject, const char *dateStr)
{
    int error;
    VALUE args[2];
    VALUE result;
    
    args[0] = conversionObject;
    args[1] = rb_str_new2(dateStr);
    result = rb_protect(wrap_conversion, (VALUE)args, &error);
    return error ? Qnil : result;
}