/*
 **  CWContainer.m
 **
 **  Copyright (c) 2002-2004
 **
 **  Author: Ludovic Marcotte <ludovic@Sophos.ca>
 **
 **  This library is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU Lesser General Public
 **  License as published by the Free Software Foundation; either
 **  version 2.1 of the License, or (at your option) any later version.
 **  
 **  This library is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 **  Lesser General Public License for more details.
 **  
 **  You should have received a copy of the GNU Lesser General Public
 **  License along with this library; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <Pantomime/CWContainer.h>

#include <Pantomime/CWConstants.h>
#include <Pantomime/CWMessage.h>

@implementation CWContainer

+ (CWContainer *)newContainer
{
    CWContainer *c;
    
    c = [[[self alloc] init] autorelease];
    return c;
}

- (id) init
{
    self = [super init];
    
    message = nil;
    parent = nil;
    child = nil;
    next = nil;
    
    return self;
}


//
//
//
- (void) dealloc
{
    [self setParent: nil];
    [self setChild: nil];
    [self setNext: nil];
    message = nil;
    [super dealloc];
}


//
// access / mutation methods
//
- (void) setParent: (CWContainer *) theParent
{
    // Resolve leaks: do not retain!
    parent = theParent;
}

- (CWContainer *)parent
{
    return parent;
}

// 
//
//
- (void) setChild: (CWContainer *) theChild
{
    if (theChild)
    {
        ASSIGN(child, theChild);
    }
    else
    {
        DESTROY(child);
    }
}

- (CWContainer *)child
{
    return child;
}

//
//
//
- (void) setNext: (CWContainer *) theNext
{
    if (theNext)
    {
        ASSIGN(next, theNext);
    }
    else
    {
        DESTROY(next);
    }
}

- (CWContainer *)next
{
    return next;
}

- (void)setMessage:(CWMessage *)aMsg
{
    [message setProperty: nil forKey: @"Container"];
    message = aMsg;
    [message setProperty: self forKey: @"Container"];

}

- (CWMessage *)message
{
    return message;
}

- (BOOL)findChild:(CWContainer *)aChild
{
#ifdef DEBUG
    if (nil != parent && nil != child)
        NSAssert(child != parent, @"Parent and child are identical!?");
#endif
    if (child == nil)
        return NO;
    else if (child == aChild)
        return YES;
    else
        return [child findChild: aChild];
}
                           
/**
 * Reverse the entire set of children
 *
 */
- (void)reverseChildren
{
    if (child != nil)
    {
        CWContainer *kid, *prev, *rest;

        for (prev = nil, kid = child, rest = [kid next];
             kid != nil;
             prev = kid, kid = rest, rest = (rest == nil ? nil : [rest next]))
        {
            [kid setNext: prev];
        }
        [self setChild: prev];
        // Do it for the kids 
        for (kid = child; kid != nil; kid = [kid next])
            [kid reverseChildren];
    }
}

- (unsigned)siblingCount
{
    CWContainer *aSibling;
    unsigned count;
    
    count = 0;
    aSibling = [self next];        
    while (aSibling)
    {
        aSibling = [aSibling next];
        count++;
    }
    return count;
}

- (unsigned)childrenCount
{
    CWContainer *aChild;
    unsigned count;
    
    count = 0;
    aChild = [self child];        
    while (aChild)
    {
        aChild = [aChild child];
        count++;
    }
    return count;
}

@end
